%% Code for Day 12

% Add path to EEGLab and launch EEGLab before we begin (or use your
% shortcut on the shortcuts bar)

    
%% ---------------------------------------------------------
% Permutation testing
% ----------------------------------------------------------

    %% Demonstration with correlation
        
        %% fabricate data for correlation for demo
        num_observations = 1000;
        x = randn(1,num_observations)';
        y = x + 9*(rand(1,length(x))');  % by using 9, this gives a population correlation of about .35, the crud factor size
        data = [x y];
        figure(1); scatter(data(:,1),data(:,2));
        true_corr=corr(data(:,1),data(:,2));
        title(['Correlation with fabricated data, ' num2str(num_observations) ' observations, r=' num2str(true_corr)]);

        %% Permutations to create distribution of r values under null hypothesis 
        
        % specify permutation parameters
        num_iters = 1000;

        % create output matrix
        r = nan(1,num_iters);

        % loop through permutations
        for n_iter = 1:num_iters
            data_shuffle = data;
            data_shuffle(:,2)= data(randperm(size(data,1)),2);
            r(n_iter) = corr(data_shuffle(:,1),data_shuffle(:,2));
        end

        % find cutoffs
        r_sort = sort(r);

        % p<.05 for positive r
        r_pos_critical = r_sort(round(length(r_sort)*.975));

        % p<.05 for negative r
        r_neg_critical = r_sort(round(length(r_sort)*.025));

        % plot the results
        figure(2); hist(r_sort);
        y_max = ylim; y_max = y_max(2); 
        line([r_pos_critical r_pos_critical],[0 y_max],'color','red');
        line([r_neg_critical r_neg_critical],[0 y_max],'color','red');
        line([true_corr true_corr],[0 y_max],'color','green');
        title(['Fabricated data permutation testing, ' num2str(num_observations) ' observations, r=' num2str(true_corr)]);

        %% Converting to Z scores
        % p-value
        pval = 0.025;  % one tailed since we have two tails
        
        % convert p-value to Z value
        z_critical = abs(norminv(pval));
        
        % compute mean and standard deviation maps
        mean_h0 = mean(r);
        std_h0  = std(r);

        % now Z-score observed correlation
        z_observed = (true_corr-mean_h0) / std_h0;

        % Is it greater than critical value?
        abs(z_observed)>z_critical

        

    %% ASSIGNMENT #1 -- investigate the parameters
        
        % Alter code above to investigate the impact of the number of iterations [100 1000 10000]
        
        % Using 1000 iterations, change sample size to be smaller or
        % larger [15 50 100 200], generate a correlation in the .3-.4 range, and examine
        % the impact on the significance testing
        
    
    %% Assignment #2: Now do with two conditions/groups
        
        %% Fabricate data
        condit1_mean = 5;
        condit2_mean = 10;
        sd1=4;
        sd2=4;
        n1=30;
        n2=30;
        
        data1= condit1_mean*ones(n1,1)+2*sd1*randn(n1,1);
        data2= condit2_mean*ones(n2,1)+2*sd2*randn(n2,1);
        
        % create histograms of the data
       figure; subplot(2,1,1)
       hist(data1);
       xlim([-10 30])
       subplot(2,1,2)
       hist(data2)
       xlim([-10 30])
       
       true_diff = mean(data1)-mean(data2);
       
       figure; hist([data1 data2]);
       figure; h1=histogram(data1,10); hold on; h2=histogram(data2,10);
        % Now set up null hypothesis and permute
            % If no difference, does not matter which condition data come
            % from
            n_iters = 1000;
            
            perm_diffs=nan(n_iters,1);
            
            for n_iter=1:n_iters
            % concatenate matrices vertically
            temp_data = [data1;data2];
            
            % derive random order for the concatenated matrix using randperm
            random_order = randperm(length(temp_data));
            temp_data = temp_data(random_order);
                       
            % now compare the mean of each permuted condition
            mean_cond1 = mean(temp_data(1:length(data1)),1);
            mean_cond2 = mean(temp_data((length(data1)+1):end),1);
            
            perm_diffs(n_iter)=mean_cond1-mean_cond2;
            
            end
            % now make a loop and do this n_iters times 
            
            % and determine whether the true mean difference is greater
            % than expected by chance at p=.05 (two tailed)
                % p-value
                pval = 0.025;  % one tailed since we have two tails
                % convert p-value to Z value
                z_critical = abs(norminv(pval));

                % compute mean and standard deviation 
                mean_diff = mean(perm_diffs);
                std_diff  = std(perm_diffs);

                % now Z-score observed 
                z_observed = (true_diff-mean_diff)/std_diff;

                % Is it greater than critical value?
                abs(z_observed)>z_critical
                
                % plot historgram of permuted diffs vs actual diff
                hist(perm_diffs);
                y_max = ylim; y_max = y_max(2); 
                line([true_diff true_diff],[0 y_max],'color','red');
                    

        
    
    %% ASSIGNMENT #3 -- determine whether P300 differs between conditions for one subject

        % load sample P300 EEG data

        load('P300.mat'); % change directory as needed

        % view the data
        pop_eegplot( EEG, 1, 1, 1);
        figure; pop_erpimage(EEG,1, [11],[[]],'PZ',10,1,{ '100' '101'},[],'type' ,'yerplabel','\muV','erp','on','cbar','on','topo', { [11] EEG.chanlocs EEG.chaninfo } );
        
        
        % create data for rare and freq
        freq_epochs=[EEG.event.type]==100;
        rare_epochs=[EEG.event.type]==101;
        
        EEG_freq_trials = EEG.data(:,:,freq_epochs);
        EEG_rare_trials = EEG.data(:,:,rare_epochs);
        
        % channel and time info
        chan2use = 'Pz';
        P3_timewin = [300 500]; % msec
        chanidx = find(strcmpi({EEG.chanlocs.labels},chan2use));
        P3timeidx = dsearchn(EEG.times',P3_timewin');
        
        % create average of rare for site Pz, all timepoints
        rare_ERP = mean(EEG_rare_trials,3);
        
        % create average of frequent for site Pz, all timepoints
        freq_ERP = mean(EEG_freq_trials,3);
        
        % plot these ERPs at PZ overlaid, with x axis labeled in msec
        figure; plot(EEG.times,rare_ERP(chanidx,:),'-r');
        hold on
        plot(EEG.times,freq_ERP(chanidx,:),'-k');
        
        % from real data, find P3 amp for each condition 
            % hint: max in P3 time window
        P3_rare=max(rare_ERP(chanidx,P3timeidx(1):P3timeidx(2)));
        P3_freq=max(freq_ERP(chanidx,P3timeidx(1):P3timeidx(2)));
        
        % Now set up null hypothesis and permute
            % Hint 1: on each iteration, re-average after shuffling trials
            % Hint 2: on each iteration, take mean on the re-averaged data
        num_iters = 1000;
        rare_means = nan(num_iters,1);
        freq_means = nan(num_iters,1);
        
        all_trials = horzcat(squeeze(EEG_rare_trials(chanidx,:,:)),squeeze(EEG_freq_trials(chanidx,:,:))  );
        for n_iter=1:num_iters
            random_order = randperm(size(all_trials,2));
            shuffle_trials = all_trials(:,random_order);
            rare_ERP_this_iteration = mean(shuffle_trials(:,1:size(EEG_rare_trials,3)),2);
            freq_ERP_this_iteration = mean(shuffle_trials(:,(size(EEG_rare_trials,3)+1):end),2);
            rare_means(n_iter) = max(rare_ERP_this_iteration(P3timeidx(1):P3timeidx(2)));
            freq_means(n_iter) = max(freq_ERP_this_iteration(P3timeidx(1):P3timeidx(2)));
        end
            % now compare the mean of each permuted condition
            perm_diffs = rare_means-freq_means;
            
            % do this n_iters times 
            
            % and determine whether the true mean difference is greater
            % than expected by chance at p=.05 (two tailed)
         % p-value
                pval = 0.025;  % one tailed since we have two tails
                % convert p-value to Z value
                z_critical = abs(norminv(pval));

                % compute mean and standard deviation 
                mean_diff = mean(perm_diffs);
                std_diff  = std(perm_diffs);

                % now Z-score observed 
                z_observed = ((P3_rare-P3_freq)-mean_diff)/std_diff;

                % Is it greater than critical value?
                abs(z_observed)>z_critical
                
        % How shall we handle the unequal n per condition?
        
        
    %% Extending to TF data
    edit permutationTesting.m
    
        
        
        
        
        
        


    
    

    